import { __decorate, __metadata } from 'tslib';
import { CommonModule } from '@angular/common';
import { TemplateRef, ElementRef, Input, Directive, EventEmitter, NgZone, HostBinding, Output, ViewChild, ContentChildren, QueryList, HostListener, Component, ViewEncapsulation, forwardRef, NgModule } from '@angular/core';
import { NG_VALUE_ACCESSOR } from '@angular/forms';

/**
 * The ToggleLabel directive allows you to customize the label for the "On" and "Off" states,
 * allowing for more robust and complex displays.
 * This directive must be used in conjunction with a ng-template.
 */
var NgToggleLabel = /** @class */ (function () {
    function NgToggleLabel(templateRef, elRef) {
        this.templateRef = templateRef;
        this.elRef = elRef;
    }
    Object.defineProperty(NgToggleLabel.prototype, "element", {
        get: function () {
            return this.elRef;
        },
        enumerable: true,
        configurable: true
    });
    NgToggleLabel.ctorParameters = function () { return [
        { type: TemplateRef },
        { type: ElementRef }
    ]; };
    __decorate([
        Input(),
        __metadata("design:type", String)
    ], NgToggleLabel.prototype, "forLabel", void 0);
    NgToggleLabel = __decorate([
        Directive({ selector: 'ng-template[ngToggleLabel]' }),
        __metadata("design:paramtypes", [TemplateRef, ElementRef])
    ], NgToggleLabel);
    return NgToggleLabel;
}());

/**
 * The Toggle directive allows for standalone or checkbox-enabled switch toggling via a UI element.
 * The toggle is styled using Bootstrap v4+ classes.
 * Accessibility implemented according to
 * https://developer.mozilla.org/en-US/docs/Web/Accessibility/ARIA/Roles/checkbox_role
 */
var NgToggle = /** @class */ (function () {
    function NgToggle(ngZone, element) {
        this.ngZone = ngZone;
        this.element = element;
        /**
         * Display text when toggled in the "On" position
         */
        this.onText = 'On';
        /**
         * Display text when toggled in the "Off" position
         */
        this.offText = 'Off';
        /**
         * Bootstrap color scheme when toggled in the "On" position (i.e. "primary" translates to "btn-primary")
         */
        this.onColor = 'primary';
        /**
         * Bootstrap color scheme when toggled in the "Off" position (i.e. "primary" translates to "btn-primary")
         */
        this.offColor = 'secondary';
        /**
         * Button size to display the toggle
         */
        this.size = '';
        /**
         * Whether the toggle is disabled or not
         */
        this.disabled = false;
        /**
         * An event fired when the user causes a change.
         * The payload of the event is the currently selected value.
         */
        this.valueChange = new EventEmitter();
        this.btnClass = true;
        this.tabindex = 0;
        this.role = 'checkbox';
        this.width = 0;
        this.handleWidth = 0;
        this._animate = true;
        this._innerAnimate = true;
        this._innerState = false;
        this._innerWidth = 'auto';
        this._disableInitialAnimation = false;
        this._dragStart = null;
        this._dragEnd = null;
        this._initialized = false;
        this._hidden = false;
    }
    Object.defineProperty(NgToggle.prototype, "outerAnimate", {
        /**
         * Optional. Enable/Disable the initial transition animation. May prevent unwanted animation display.
         */
        set: function (value) {
            this._disableInitialAnimation = value;
        },
        enumerable: true,
        configurable: true
    });
    Object.defineProperty(NgToggle.prototype, "value", {
        get: function () {
            return this._innerState;
        },
        set: function (value) {
            this.setState(value);
        },
        enumerable: true,
        configurable: true
    });
    NgToggle.prototype.ngAfterViewInit = function () {
        var _this = this;
        this.calculateWidth();
        this._initialized = true;
        this.ngZone.run(function () { return setTimeout(function () { return _this._disableInitialAnimation = false; }); });
    };
    NgToggle.prototype.ngAfterViewChecked = function () {
        var hidden = this.element.nativeElement.offsetParent === null;
        if ((!this._initialized || this.width === 0) && this._hidden !== hidden) {
            this._initialized = false;
            this.calculateWidth();
            this._initialized = true;
        }
        this._hidden = hidden;
    };
    NgToggle.prototype.ngAfterContentInit = function () {
        var onElement = this.labelElements.find(function (item) { return item.forLabel.toLowerCase() === 'on'; });
        var offElement = this.labelElements.find(function (item) { return item.forLabel.toLowerCase() === 'off'; });
        if (onElement) {
            this.onLabel = onElement;
        }
        if (offElement) {
            this.offLabel = offElement;
        }
        if (onElement || offElement) {
            this.calculateWidth(true);
        }
    };
    NgToggle.prototype.ngOnChanges = function (changes) {
        if ('onText' in changes || 'offText' in changes || 'size' in changes) {
            this._initialized = false;
            this.calculateWidth(this._initialized);
            this._initialized = true;
        }
    };
    Object.defineProperty(NgToggle.prototype, "largeButton", {
        get: function () {
            return this.size === 'lg';
        },
        enumerable: true,
        configurable: true
    });
    Object.defineProperty(NgToggle.prototype, "smallButton", {
        get: function () {
            return this.size === 'sm';
        },
        enumerable: true,
        configurable: true
    });
    Object.defineProperty(NgToggle.prototype, "handleDark", {
        get: function () {
            return (this.value && this.onColor === 'light') || (!this.value && this.offColor === 'light');
        },
        enumerable: true,
        configurable: true
    });
    Object.defineProperty(NgToggle.prototype, "indeterminate", {
        get: function () {
            return this._innerState === null || typeof this._innerState === 'undefined';
        },
        enumerable: true,
        configurable: true
    });
    Object.defineProperty(NgToggle.prototype, "toggledOn", {
        get: function () {
            return this.innerState === true;
        },
        enumerable: true,
        configurable: true
    });
    Object.defineProperty(NgToggle.prototype, "toggledOff", {
        get: function () {
            return this.innerState === false;
        },
        enumerable: true,
        configurable: true
    });
    Object.defineProperty(NgToggle.prototype, "innerState", {
        get: function () {
            return this._innerState;
        },
        enumerable: true,
        configurable: true
    });
    Object.defineProperty(NgToggle.prototype, "ariaCheckedValue", {
        get: function () {
            if (this.indeterminate) {
                return 'mixed';
            }
            return this.value ? 'true' : 'false';
        },
        enumerable: true,
        configurable: true
    });
    Object.defineProperty(NgToggle.prototype, "animate", {
        get: function () {
            return this._animate && (!this._disableInitialAnimation || this._initialized && !this._disableInitialAnimation);
        },
        enumerable: true,
        configurable: true
    });
    Object.defineProperty(NgToggle.prototype, "marginLeft", {
        get: function () {
            var margin = 0;
            if (!this._initialized) {
            }
            else if (this.indeterminate || this._innerState === null || typeof this._innerState === 'undefined') {
                margin = -(this.width / 2);
            }
            else if (this._dragEnd) {
                margin = this._dragEnd;
            }
            else if (!this._innerState) {
                margin = -this.width;
            }
            return margin + 'px';
        },
        enumerable: true,
        configurable: true
    });
    NgToggle.prototype.handleResize = function () {
        this._initialized = false;
        this.calculateWidth(true);
        this._initialized = true;
    };
    NgToggle.prototype.handleClick = function () {
        if (!this.disabled && !this._dragEnd) {
            this.setState(!this._innerState);
        }
        else if (this._dragEnd) {
            this._dragEnd = null;
        }
    };
    NgToggle.prototype.onTouchStart = function (event) {
        this.onDragStart(event);
    };
    NgToggle.prototype.onMouseDown = function (event) {
        this.onDragStart(event);
    };
    NgToggle.prototype.onTouchMove = function (event) {
        this.onDragMove(event);
    };
    NgToggle.prototype.onMouseMove = function (event) {
        this.onDragMove(event);
    };
    NgToggle.prototype.onTouchEnd = function (event) {
        this.onDragEnd(event, true);
    };
    NgToggle.prototype.onMouseUp = function (event) {
        this.onDragEnd(event);
    };
    NgToggle.prototype.onMouseLeave = function (event) {
        this.onDragEnd(event, true);
    };
    NgToggle.prototype.onKeyDown = function (event) {
        if (!event.key || this.disabled) {
            return;
        }
        switch (event.key) {
            case 'Left':
            case 'ArrowLeft':
                event.preventDefault();
                event.stopImmediatePropagation();
                this.setState(false);
                break;
            case 'Right':
            case 'ArrowRight':
                event.preventDefault();
                event.stopImmediatePropagation();
                this.setState(true);
                break;
            case 'Spacebar':
            case ' ':
                event.preventDefault();
                event.stopImmediatePropagation();
                this.setState(!this.value);
        }
    };
    NgToggle.prototype.onDragStart = function (event) {
        if (event.target === this.handle$) {
            if (this._dragStart || this.disabled) {
                return;
            }
            event.preventDefault();
            event.stopPropagation();
            this._dragStart = (event.pageX || event.touches[0].pageX) - parseInt(this.container$.style.marginLeft, 10);
            if (this._animate) {
                this._animate = !this._animate;
            }
        }
    };
    NgToggle.prototype.onDragMove = function (event) {
        if (this._dragStart) {
            event.preventDefault();
            var difference = (event.pageX || event.touches[0].pageX) - this._dragStart;
            if (difference < -(Number(this.width)) || difference > 0) {
                return;
            }
            this._dragEnd = difference;
        }
    };
    NgToggle.prototype.onDragEnd = function (event, clearDragEnd) {
        if (clearDragEnd === void 0) { clearDragEnd = false; }
        if (this._dragStart) {
            event.preventDefault();
            event.stopPropagation();
            if (this._dragEnd) {
                this.setState(this._dragEnd > -(Number(this.width) / 2));
            }
            this._dragStart = null;
            if (clearDragEnd) {
                this._dragEnd = null;
            }
            if (this._innerAnimate && !this._animate) {
                this._animate = true;
            }
        }
    };
    NgToggle.prototype.calculateWidth = function (disableAnimation) {
        var _this = this;
        if (disableAnimation === void 0) { disableAnimation = false; }
        if (disableAnimation && this._innerAnimate) {
            this._animate = false;
        }
        var initialized = this._initialized;
        if (!initialized) {
            this.container$.style.width = 'auto';
            this.element$.style.width = 'auto';
        }
        this.on$.style.width = 'auto';
        this.off$.style.width = 'auto';
        setTimeout(function () {
            var width = _this._innerWidth;
            if (_this._innerWidth === 'auto') {
                width = Math.max(_this.on$.offsetWidth, _this.off$.offsetWidth);
            }
            _this.handleWidth = _this.handle$.offsetWidth;
            _this.width = Number(width);
            if (!initialized) {
                _this.container$.style.width = ((_this.width * 2) + _this.handleWidth) + 'px';
                _this.element$.style.width = (_this.width + _this.handleWidth) + 'px';
            }
            _this.ngZone.run(function () {
                _this.on$.style.width = _this.width + 'px';
                _this.off$.style.width = _this.width + 'px';
                setTimeout(function () {
                    if (disableAnimation && _this._innerAnimate) {
                        _this._animate = true;
                    }
                });
            });
        });
    };
    NgToggle.prototype.setState = function (value) {
        if (value !== this._innerState) {
            this._innerState = value;
            this.valueChange.emit(this._innerState);
        }
    };
    Object.defineProperty(NgToggle.prototype, "element$", {
        get: function () {
            return this.element.nativeElement;
        },
        enumerable: true,
        configurable: true
    });
    Object.defineProperty(NgToggle.prototype, "on$", {
        get: function () {
            return this.onElement.nativeElement;
        },
        enumerable: true,
        configurable: true
    });
    Object.defineProperty(NgToggle.prototype, "off$", {
        get: function () {
            return this.offElement.nativeElement;
        },
        enumerable: true,
        configurable: true
    });
    Object.defineProperty(NgToggle.prototype, "handle$", {
        get: function () {
            return this.handleElement.nativeElement;
        },
        enumerable: true,
        configurable: true
    });
    Object.defineProperty(NgToggle.prototype, "container$", {
        get: function () {
            return this.containerElement.nativeElement;
        },
        enumerable: true,
        configurable: true
    });
    NgToggle.ctorParameters = function () { return [
        { type: NgZone },
        { type: ElementRef }
    ]; };
    __decorate([
        Input(),
        __metadata("design:type", String)
    ], NgToggle.prototype, "onText", void 0);
    __decorate([
        Input(),
        __metadata("design:type", String)
    ], NgToggle.prototype, "offText", void 0);
    __decorate([
        Input(),
        __metadata("design:type", String)
    ], NgToggle.prototype, "onColor", void 0);
    __decorate([
        Input(),
        __metadata("design:type", String)
    ], NgToggle.prototype, "offColor", void 0);
    __decorate([
        Input(),
        __metadata("design:type", String)
    ], NgToggle.prototype, "size", void 0);
    __decorate([
        Input(), HostBinding('class.disabled'), HostBinding('class.ng-toggle-disabled'),
        __metadata("design:type", Boolean)
    ], NgToggle.prototype, "disabled", void 0);
    __decorate([
        Input('disableInitialAnimation'),
        __metadata("design:type", Boolean),
        __metadata("design:paramtypes", [Boolean])
    ], NgToggle.prototype, "outerAnimate", null);
    __decorate([
        Input(),
        __metadata("design:type", Boolean),
        __metadata("design:paramtypes", [Boolean])
    ], NgToggle.prototype, "value", null);
    __decorate([
        Output(),
        __metadata("design:type", EventEmitter)
    ], NgToggle.prototype, "valueChange", void 0);
    __decorate([
        HostBinding('class.btn'),
        __metadata("design:type", Boolean)
    ], NgToggle.prototype, "btnClass", void 0);
    __decorate([
        Input('tabindex'), HostBinding('attr.tabindex'),
        __metadata("design:type", Number)
    ], NgToggle.prototype, "tabindex", void 0);
    __decorate([
        Input('role'), HostBinding('attr.role'),
        __metadata("design:type", Object)
    ], NgToggle.prototype, "role", void 0);
    __decorate([
        ViewChild('container', { static: true }),
        __metadata("design:type", ElementRef)
    ], NgToggle.prototype, "containerElement", void 0);
    __decorate([
        ViewChild('on', { static: true }),
        __metadata("design:type", ElementRef)
    ], NgToggle.prototype, "onElement", void 0);
    __decorate([
        ViewChild('off', { static: true }),
        __metadata("design:type", ElementRef)
    ], NgToggle.prototype, "offElement", void 0);
    __decorate([
        ViewChild('handle', { static: true }),
        __metadata("design:type", ElementRef)
    ], NgToggle.prototype, "handleElement", void 0);
    __decorate([
        ContentChildren(NgToggleLabel),
        __metadata("design:type", QueryList)
    ], NgToggle.prototype, "labelElements", void 0);
    __decorate([
        HostBinding('class.btn-lg'),
        HostBinding('class.ng-toggle-lg'),
        __metadata("design:type", Boolean),
        __metadata("design:paramtypes", [])
    ], NgToggle.prototype, "largeButton", null);
    __decorate([
        HostBinding('class.btn-sm'),
        HostBinding('class.ng-toggle-sm'),
        __metadata("design:type", Boolean),
        __metadata("design:paramtypes", [])
    ], NgToggle.prototype, "smallButton", null);
    __decorate([
        HostBinding('class.ng-toggle-indeterminate'),
        __metadata("design:type", Boolean),
        __metadata("design:paramtypes", [])
    ], NgToggle.prototype, "indeterminate", null);
    __decorate([
        HostBinding('class.ng-toggled-on'),
        __metadata("design:type", Boolean),
        __metadata("design:paramtypes", [])
    ], NgToggle.prototype, "toggledOn", null);
    __decorate([
        HostBinding('class.ng-toggled-off'),
        __metadata("design:type", Boolean),
        __metadata("design:paramtypes", [])
    ], NgToggle.prototype, "toggledOff", null);
    __decorate([
        HostBinding('attr.aria-checked'),
        __metadata("design:type", String),
        __metadata("design:paramtypes", [])
    ], NgToggle.prototype, "ariaCheckedValue", null);
    __decorate([
        HostBinding('class.ng-toggle-animate'),
        __metadata("design:type", Boolean),
        __metadata("design:paramtypes", [])
    ], NgToggle.prototype, "animate", null);
    __decorate([
        HostListener('window:resize'),
        __metadata("design:type", Function),
        __metadata("design:paramtypes", []),
        __metadata("design:returntype", void 0)
    ], NgToggle.prototype, "handleResize", null);
    __decorate([
        HostListener('click'),
        __metadata("design:type", Function),
        __metadata("design:paramtypes", []),
        __metadata("design:returntype", void 0)
    ], NgToggle.prototype, "handleClick", null);
    __decorate([
        HostListener('touchstart', ['$event']),
        __metadata("design:type", Function),
        __metadata("design:paramtypes", [Object]),
        __metadata("design:returntype", void 0)
    ], NgToggle.prototype, "onTouchStart", null);
    __decorate([
        HostListener('mousedown', ['$event']),
        __metadata("design:type", Function),
        __metadata("design:paramtypes", [Object]),
        __metadata("design:returntype", void 0)
    ], NgToggle.prototype, "onMouseDown", null);
    __decorate([
        HostListener('touchmove', ['$event']),
        __metadata("design:type", Function),
        __metadata("design:paramtypes", [Object]),
        __metadata("design:returntype", void 0)
    ], NgToggle.prototype, "onTouchMove", null);
    __decorate([
        HostListener('mousemove', ['$event']),
        __metadata("design:type", Function),
        __metadata("design:paramtypes", [Object]),
        __metadata("design:returntype", void 0)
    ], NgToggle.prototype, "onMouseMove", null);
    __decorate([
        HostListener('touchend', ['$event']),
        __metadata("design:type", Function),
        __metadata("design:paramtypes", [Object]),
        __metadata("design:returntype", void 0)
    ], NgToggle.prototype, "onTouchEnd", null);
    __decorate([
        HostListener('mouseup', ['$event']),
        __metadata("design:type", Function),
        __metadata("design:paramtypes", [Object]),
        __metadata("design:returntype", void 0)
    ], NgToggle.prototype, "onMouseUp", null);
    __decorate([
        HostListener('mouseleave', ['$event']),
        __metadata("design:type", Function),
        __metadata("design:paramtypes", [Object]),
        __metadata("design:returntype", void 0)
    ], NgToggle.prototype, "onMouseLeave", null);
    __decorate([
        HostListener('keydown', ['$event']),
        __metadata("design:type", Function),
        __metadata("design:paramtypes", [KeyboardEvent]),
        __metadata("design:returntype", void 0)
    ], NgToggle.prototype, "onKeyDown", null);
    NgToggle = __decorate([
        Component({
            selector: 'ng-toggle',
            template: "\n    <span #container class=\"ng-toggle-container\" [style.marginLeft]=\"marginLeft\">\n      <span #on class=\"ng-toggle-on btn btn-{{onColor}}\" [class.btn-lg]=\"largeButton\"\n            [class.btn-sm]=\"smallButton\" [class.disabled]=\"disabled\">\n          <ng-template [ngTemplateOutlet]=\"onLabel?.templateRef\"></ng-template>\n          <ng-container *ngIf=\"!onLabel\">{{onText}}</ng-container>\n      </span>\n      <span #handle class=\"ng-toggle-handle btn\" [class.btn-lg]=\"largeButton\"\n            [class.btn-sm]=\"smallButton\" [class.disabled]=\"disabled\"\n            [class.btn-light]=\"!handleDark\" [class.btn-dark]=\"handleDark\">&nbsp;</span>\n      <span #off class=\"ng-toggle-off btn btn-{{offColor}}\" [class.btn-lg]=\"largeButton\"\n            [class.btn-sm]=\"smallButton\" [class.disabled]=\"disabled\">\n        <ng-template [ngTemplateOutlet]=\"offLabel?.templateRef\"></ng-template>\n        <ng-container *ngIf=\"!offLabel\">{{offText}}</ng-container>\n      </span>\n    </span>\n    <ng-content></ng-content>\n    ",
            preserveWhitespaces: false,
            encapsulation: ViewEncapsulation.None,
            styles: ["ng-toggle{position:relative;display:inline-block;direction:ltr;cursor:pointer;overflow:hidden;padding:0!important;text-align:left;z-index:0;-webkit-user-select:none;-moz-user-select:none;-ms-user-select:none;user-select:none;vertical-align:middle;transition:border-color .15s ease-in-out,box-shadow .15s ease-in-out;box-sizing:content-box}ng-toggle.disabled,ng-toggle.disabled .btn{cursor:default}ng-toggle input{position:absolute;z-index:-1;visibility:hidden;width:1px;height:1px}ng-toggle .ng-toggle-container,ng-toggle .ng-toggle-handle,ng-toggle .ng-toggle-off,ng-toggle .ng-toggle-on{display:-ms-flexbox!important;display:flex!important}ng-toggle .ng-toggle-container{-ms-flex-align:stretch!important;align-items:stretch!important;top:0;border-radius:0;-webkit-transform:translateZ(0);transform:translateZ(0)}ng-toggle.ng-toggle-animate .ng-toggle-container{transition:margin-left .5s}ng-toggle .ng-toggle-off,ng-toggle .ng-toggle-on{-ms-flex-align:center!important;align-items:center!important;text-align:center;z-index:1;border-radius:0}ng-toggle .ng-toggle-handle,ng-toggle .ng-toggle-off,ng-toggle .ng-toggle-on{box-sizing:border-box;cursor:pointer;-webkit-user-select:none;-moz-user-select:none;-ms-user-select:none;user-select:none}ng-toggle .ng-toggle-handle{text-align:center;margin-top:-1px;margin-bottom:-1px;z-index:100;width:1em;padding-left:0;padding-right:0;-ms-flex-item-align:stretch!important;-ms-grid-row-align:stretch!important;align-self:stretch!important}"]
        }),
        __metadata("design:paramtypes", [NgZone, ElementRef])
    ], NgToggle);
    return NgToggle;
}());

var NgToggleAccessor = /** @class */ (function () {
    function NgToggleAccessor(_host) {
        var _this = this;
        this._host = _host;
        this._subscription = this._host.valueChange.subscribe(function (value) { return _this.onChange(value); });
    }
    NgToggleAccessor_1 = NgToggleAccessor;
    NgToggleAccessor.prototype.ngOnDestroy = function () {
        if (this._subscription) {
            this._subscription.unsubscribe();
            this._subscription = null;
        }
    };
    NgToggleAccessor.prototype.onChange = function (_) {
        if (this._onChange) {
            this._onChange(this._host.value);
        }
    };
    NgToggleAccessor.prototype.onTouched = function () {
        if (this._onTouched) {
            this._onTouched();
        }
    };
    NgToggleAccessor.prototype.registerOnChange = function (fn) {
        this._onChange = fn;
    };
    NgToggleAccessor.prototype.registerOnTouched = function (fn) {
        this._onTouched = fn;
    };
    NgToggleAccessor.prototype.setDisabledState = function (isDisabled) {
        this._host.disabled = isDisabled;
    };
    NgToggleAccessor.prototype.writeValue = function (obj) {
        if (typeof obj === 'boolean' || obj === null) {
            this._host.value = obj;
        }
    };
    var NgToggleAccessor_1;
    NgToggleAccessor.ctorParameters = function () { return [
        { type: NgToggle }
    ]; };
    NgToggleAccessor = NgToggleAccessor_1 = __decorate([
        Directive({
            selector: 'ng-toggle',
            host: { '(change)': 'onChange($event)', '(touch)': 'onTouched()' },
            providers: [{ provide: NG_VALUE_ACCESSOR, useExisting: forwardRef(function () { return NgToggleAccessor_1; }), multi: true }]
        }),
        __metadata("design:paramtypes", [NgToggle])
    ], NgToggleAccessor);
    return NgToggleAccessor;
}());

var declarations = [
    NgToggle,
    NgToggleAccessor,
    NgToggleLabel,
];
var NgToggleModule = /** @class */ (function () {
    function NgToggleModule() {
    }
    NgToggleModule = __decorate([
        NgModule({ imports: [CommonModule], exports: declarations, declarations: declarations })
    ], NgToggleModule);
    return NgToggleModule;
}());

/**
 * Generated bundle index. Do not edit.
 */

export { NgToggle, NgToggleAccessor, NgToggleLabel, NgToggleModule };
//# sourceMappingURL=ng-toggle.js.map
